from os import environ
import sys

from boto.dynamodb import connect_to_region

from .backend import Backend
from .frontend import PdnsFrontend

def get_aws_credentials(params):
    access_key=params.get('--access-key-id') or environ.get('AWS_ACCESS_KEY_ID')
    if params.get('--secret-key-file'):
        with open(params.get('--secret-key-file')) as f:
            secret_key = f.read().strip()
    else:
        secret_key = params.get('--secret-key-id') or environ.get('AWS_SECRET_ACCESS_KEY')
    return access_key, secret_key

def exit_with_error(error):
    sys.stderr.write(error)
    sys.exit(1)

def get_dynamodb_connection(params):
    region = params['--region']
    access_key, secret_key = get_aws_credentials(params)
    if not (access_key and secret_key):
        exit_with_error('ERROR: Invalid AWS credentials supplied.')
    connection = connect_to_region(region,
                                   aws_access_key_id=access_key,
                                   aws_secret_access_key=secret_key)
    if not connection:
        exit_with_error('ERROR: unable to connect, check your region')

    return connection

def run(params):
    connection = get_dynamodb_connection(params)
    backend = Backend(connection=connection, table_name=params['--table'])
    frontend = PdnsFrontend(sys.stdin, sys.stdout, backend)
    frontend.expect_helo()
    frontend.run()



from pdns_dynamodb import __version__

ABI_VERSION = '1'

class PdnsFrontend(object):
    def __init__(self, input_stream, output_stream, backend):
        self.input_stream = input_stream
        self.output_stream = output_stream
        self.backend = backend
        self.logs = []

    def expect_helo(self):
        line = self.readline()
        if not line.startswith('HELO\t'):
            self.fail('invalid HELO line')
            self.readline()
        else:
            _, abi = line.split('\t')
            if abi == ABI_VERSION:
                self.reply('OK\tpdns-dynamodb version %s' % __version__)
            else:
                self.fail('got abi-version %s, expected %s' % (abi, ABI_VERSION))
                self.readline()

    def readline(self):
        return self.input_stream.readline().strip()

    def reply(self, message):
        self.output_stream.write(message)
        self.output_stream.write('\n')
        self.output_stream.flush()

    def log(self, msg):
        self.logs.append(msg)

    def send_logs(self):
        for log in self.logs:
            self.reply('LOG\t%s' % log)
        self.logs = []

    def end(self):
        self.send_logs()
        self.reply('END')

    def fail(self, msg=None):
        self.send_logs()
        if msg:
            self.reply('FAIL\t%s' % msg)
        else:
            self.reply('FAIL')

    def output_record(self, record):
        # DATA qname qclass qtype ttl id content
        msg = '\t'.join(['DATA', record['name'], record['qclass'],
                   record['qtype'], str(record['ttl']), '-1', record['content']])
        self.reply(msg)

    def parse_query(self, line):
        query = line.split("\t")  # Q, qname, qclass, qtype, id, remote-ip-address
        return dict(qname=query[1], qclass=query[2], qtype=query[3])

    def run_one_query(self):
        line = self.readline()
        if not line:
            return
        query = self.parse_query(line)
        if query['qclass'] != 'IN':
            self.log('Class %s not supported' % qclass)
            self.fail()
        else:
            records = self.backend.query(query['qname'], query['qtype'])
            for record in records:
                self.output_record(record)
            self.end()

    def run(self):
        while True:
            self.run_one_query()



import os
from setuptools import setup

from pdns_dynamodb import __version__

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='pdns-dynamodb',
      version=__version__,
      description='PowerDNS backend using DynamoDB',
      long_description=read('README.rst'),
      author='Cosmin Stejerean',
      author_email='cosmin@offbytwo.com',
      license='Apache License 2.0',
      url='http://github.com/cosmin/pdns-dynamodb',
      packages=['pdns_dynamodb'],
      scripts=['bin/pdns-dynamodb'],
      tests_require=open('test-requirements.txt').readlines(),
      install_requires=open('requirements.txt').readlines(),
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Utilities'
        ]
     )



import unittest
from mock import Mock, call
from boto.dynamodb.condition import EQ

from pdns_dynamodb.backend import Backend

class TestBackend(unittest.TestCase):
    def setUp(self):
        self.con = Mock()
        self.table_name = 'SOME_TABLE'
        self.backend = Backend(self.con, self.table_name)

    def test_init_retrieves_the_table(self):
        self.con.get_table.assert_called_once_with(self.table_name)

    def test_query_params_for_ANY_query(self):
        ret = self.backend._query_params_from_query_type('ANY')
        self.assertEqual(ret, {})

    def test_query_params_for_other_queries(self):
        ret = self.backend._query_params_from_query_type('CNAME')
        self.assertEqual(ret, {'range_key_condition':EQ('CNAME')})

    def test_run_regular_query_against_table(self):
        self.backend.table.query.return_value = []
        self.backend.query('example.com', 'NS')
        table = self.backend.table
        table.query.assert_called_once_with('example.com',
                                            range_key_condition=EQ('NS'))

    def test_record_from_value(self):
        name = 'example.com'
        item = {'name': name, 'type':'A', 'ttl': 3600, 'values': ['1.1.1.1']}
        record = self.backend._record_from_value('example.com', item, '1.1.1.1')
        self.assertEqual(record, dict(name=name, qtype='A', qclass='IN',
                                      ttl=3600, content='1.1.1.1'))

    def test_query_builds_records_from_values(self):
        table = self.backend.table
        name = 'example.com'
        item = {'name': name, 'type':'A', 'ttl': 3600, 'values': ['1.1.1.1',
                                                                  '2.2.2.2']}
        table.query.return_value = [item]
        records = self.backend.query(name)
        self.assertEqual(records, [dict(name=name, qtype='A', qclass='IN',
                                      ttl=3600, content='1.1.1.1'),
                                   dict(name=name, qtype='A', qclass='IN',
                                      ttl=3600, content='2.2.2.2')])



import unittest
from mock import Mock, call

from pdns_dynamodb.frontend import *
from pdns_dynamodb.version import __version__

class TestFrontend(unittest.TestCase):
    def setUp(self):
        self.input = Mock()
        self.output = Mock()
        self.backend = Mock()
        self.frontend = PdnsFrontend(self.input, self.output, self.backend)

    def assert_reply(self, expected):
        self.assert_replies(expected)

    def assert_replies(self, *replies):
        expected = []
        for reply in replies:
            expected.append(call(reply))
            expected.append(call('\n'))
        self.output.write.assert_has_calls(expected)

    def test_unsupported_abi(self):
        self.input.readline.return_value = 'HELO\t0'
        self.frontend.expect_helo()
        self.assert_reply('FAIL\tgot abi-version 0, expected 1')

    def test_invalid_helo(self):
        self.input.readline.return_value = 'JUNK'
        self.frontend.expect_helo()
        self.assert_reply('FAIL\tinvalid HELO line')

    def test_valid_helo(self):
        self.input.readline.return_value = 'HELO\t1'
        self.frontend.expect_helo()
        self.assert_reply('OK\tpdns-dynamodb version %s' % __version__)

    def test_parsing_the_query(self):
        data = self.frontend.parse_query('Q\texample.com\tIN\tANY\t-1\t0.0.0.0')
        self.assertEqual(data['qname'], 'example.com')
        self.assertEqual(data['qclass'], 'IN')
        self.assertEqual(data['qtype'], 'ANY')

    def test_output_record(self):
        record = dict(name='example.com', qclass='IN', qtype='A',
                      ttl=3600, content='1.1.1.1')
        self.frontend.output_record(record)
        self.assert_reply('DATA\texample.com\tIN\tA\t3600\t-1\t1.1.1.1')

    def test_run_one_query(self):
        self.input.readline.return_value = 'Q\texample.com\tIN\tANY\t-1\t0.0.0.0'
        records = [dict(name='example.com', qclass='IN', qtype='A',
                       ttl=3600, content='1.1.1.1'),
                   dict(name='example.com', qclass='IN', qtype='NS',
                        ttl=4800, content='ns1.example.com')]
        self.backend.query.return_value = records
        self.frontend.log('hello, world')
        self.frontend.run_one_query()
        self.backend.query.callend_once_with('example.com', 'ANY')
        self.assert_replies('DATA\texample.com\tIN\tA\t3600\t-1\t1.1.1.1',
                            'DATA\texample.com\tIN\tNS\t4800\t-1\tns1.example.com',
                            'LOG\thello, world',
                            'END')



from boto.dynamodb import connect_to_region
from boto.dynamodb.condition import EQ
from boto.dynamodb.exceptions import DynamoDBKeyNotFoundError

class Backend(object):
    def __init__(self, connection, table_name):
        self.con = connection
        self.table = self.con.get_table(table_name)

    def _query_params_from_query_type(self, query_type):
        if query_type == 'ANY':
            return {}
        else:
            return dict(range_key_condition=EQ(query_type))

    def _record_from_value(self, name, item, value):
        return dict(name=name, qtype=item['type'], qclass='IN',
                    ttl=item['ttl'], content=value)

    def query(self, name, query_type='ANY'):
        records = []
        query_params = self._query_params_from_query_type(query_type)
        for item in self.table.query(name, **query_params):
            for value in item['values']:
                records.append(self._record_from_value(name, item, value))
        return records
